from django.db import models
import boto.ec2


class AWSNotFoundException(Exception):
    """Server not found at AWS"""

class AWSServer(models.Model):
    """
    AWSServer represents a single EC2 server at AWS
    """
    name = models.CharField(max_length=20)
    description = models.CharField(max_length=100, null=True, blank=True)
    aws_access_key = models.CharField(max_length=20)
    aws_secret_key = models.CharField(max_length=40)
    aws_region = models.CharField(max_length=20, default='us-east-1')
    user_name = models.CharField(max_length=20, default='Administrator')

    def __unicode__(self):
        return self.name

    def get_connection(self):
        """
        Connects to AWS and returns and instance
        """
        conn = boto.ec2.connect_to_region(self.aws_region, aws_access_key_id=self.aws_access_key, aws_secret_access_key=self.aws_secret_key)
        all_instances = conn.get_all_instances(filters={'tag:Name': self.name})
        if not all_instances:
            raise AWSNotFoundException("AWS Instance not found, check settings.")
        inst = all_instances[0].instances[0]
        return inst

    def start_server(self):
        """
        Sends a message to AWS to start the server
        """
        inst = self.get_connection()
        inst.start()

    def stop_server(self):
        """
        Sends a message to AWS to stop the server
        """
        inst = self.get_connection()
        inst.stop()

    def get_server_state(self):
        """
        returns the state of the server (e.g., 'running', 'stopped')
        """
        inst = self.get_connection()
        return inst.state




"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
import os
from django.test import TestCase
from models import AWSServer

class AWSServerTest(TestCase):

    def setUp(self):
        # save aws keys in environment for testing (so as to not put them in public github)
        TEST_AWS_NAME = os.environ.get("TEST_AWS_NAME")
        TEST_AWS_ACCESS_KEY = os.environ.get("TEST_AWS_ACCESS_KEY")
        TEST_AWS_SECRET_KEY = os.environ.get("TEST_AWS_SECRET_KEY")
        TEST_AWS_REGION = os.environ.get("TEST_AWS_REGION")
        TEST_AWS_USER_NAME = os.environ.get("TEST_AWS_USER_NAME")

        # make sure env vars are set
        self.assertIsNotNone(TEST_AWS_ACCESS_KEY)
        self.assertIsNotNone(TEST_AWS_SECRET_KEY)
        self.assertIsNotNone(TEST_AWS_REGION)
        self.assertIsNotNone(TEST_AWS_USER_NAME)

        test_server = AWSServer(
            name = TEST_AWS_NAME,
            description = "Test AWS Server",
            aws_access_key = TEST_AWS_ACCESS_KEY,
            aws_secret_key = TEST_AWS_SECRET_KEY,
            aws_region = TEST_AWS_REGION,
            user_name = TEST_AWS_USER_NAME,
        )
        test_server.save()

    def test_server_state(self):
        """ connect to the server and get it's state """
        test_server = AWSServer.objects.get(pk=1)
        state = test_server.get_server_state()
        self.assertIn(state, ("running", "stopped", "shutting-down", "terminated", "stopping"))

    def test_server_name_incorrect(self):
        """ Test that an incorrect server name will give the appropriate error msg """
        test_server = AWSServer.objects.get(pk=1)
        test_server.name = "wrongname"
        try:
            test_server.get_server_state()
        except Exception as e:
            msg = str(e)

        self.assertEqual(msg, "AWS Instance not found, check settings.")




from django.contrib import admin
from django.contrib import messages

from django.utils.html import format_html
import boto.ec2

class AWSServerAdmin(admin.ModelAdmin):

    list_display = ('name', 'description', 'user_name','aws_region')

    state = None
    public_dns_name = None
    server_status_info = None
    inst = None

    def get_inst_state(self, instance):
        return self.state
    get_inst_state.short_description  = "Instance State"

    def get_public_dns_name(self, instance):
        return self.public_dns_name
    get_public_dns_name.short_description  = "Public DNS Name"

    def get_rdp_url(self, instance):
        if self.state == "running":
            return format_html('<a href="/aws_manager/get_rdp/{0}/{1}">RDP File</a>', self.public_dns_name, instance.user_name)
        else:
            return 'Not Available (server not running)'

    get_rdp_url.short_description  = "Remote Desktop File"
    get_rdp_url.allow_tags = True


    def change_view(self, request, object_id, form_url='', extra_context=None):
        try:
            server = self.get_object(self, object_id)
            conn = boto.ec2.connect_to_region(server.aws_region, aws_access_key_id=server.aws_access_key, aws_secret_access_key=server.aws_secret_key)
            inst = conn.get_all_instances(filters={'tag:Name': server.name})[0].instances[0]
            self.state = inst.state
            self.public_dns_name = inst.public_dns_name
            self.readonly_fields = (self.get_inst_state, self.get_public_dns_name, self.get_rdp_url)
        except:
            self.readonly_fields = ()
            messages.add_message(request, messages.ERROR, 'Warning: Server not found.  Please check AWS connection details')

        return super(AWSServerAdmin, self).change_view(request, object_id,
            form_url, extra_context=extra_context)

    def action_start_server(self, request, queryset):
        for obj in queryset:
            try:
                obj.start_server()
            except Exception as exc:
                self.message_user(request, "Error: can't start " + obj.name + " server. %s" % exc, level=messages.ERROR)
        self.message_user(request, "%s AWS servers sent start command." % queryset.count())

    action_start_server.short_description = "Start AWS Server"

    def action_stop_server(self, request, queryset):
        for obj in queryset:
            try:
                obj.stop_server()
            except Exception as exc:
                self.message_user(request, "Error: can't stop " + obj.name + " server. %s" % exc, level=messages.ERROR)
        self.message_user(request, "%s AWS servers sent stop command." % queryset.count())

    action_stop_server.short_description = "Stop AWS Server"

    actions = ['action_start_server', 'action_stop_server']








from aws_manager import views

# DJANGO IMPORTS
try:
    from django.conf.urls import url, patterns
except ImportError:
    # for Django version less then 1.4
    from django.conf.urls.defaults import url, patterns

urlpatterns = patterns('',
    url(r'^get_rdp/(?P<public_dns>[\w|\W]+)/(?P<username>\w+)/$', 'aws_manager.views.get_rdp'),
)




from django.http import HttpResponse

def get_rdp(request, public_dns,username):
    # send a remote desktop file to user
    rdp_content = "auto connect:i:1\nfull address:s:%s\nusername:s:%s" % (public_dns, username)
    response = HttpResponse(rdp_content, mimetype='application/x-rdp')
    response['Content-Disposition'] = 'attachment; filename="ph_server_connect.rdp"'
    return response

